#!/usr/bin/env python

import sys, math, traceback, random, time
import subprocess
import getopt
import os

from net import Socket
from polje import Polje

def vir_rand(**kwargs):
    if not kwargs['N'] or not kwargs['W'] or not kwargs['H'] or not kwargs['M']:
        print "Manjkajoci parametri. (podani: " + str(dict((a,b) for a,b in kwargs.viewitems() if b)) + ")"
        return
    for i in range(kwargs['N']):
        p = Polje(w=kwargs['W'], h=kwargs['H'], m=kwargs['M'])
        if kwargs['opt_pfx']: p.dump(kwargs['opt_pfx'] + '_' + str(i) + '.mp')
        yield p

def vir_fnm(**kwargs):
    yield Polje(fnm=kwargs['fnm'])

def vir_dir(**kwargs):
    for fnm in os.listdir(kwargs['dir']):
        if fnm[-3:] != ".mp": continue
        yield Polje(fnm=fnm)

def igra(c, cli_ime, polje, dump=False):
    "Izvede eno igro."
    if dump:
        print "Igralno polje: (%d x %d)" % (polje.w, polje.h)
        polje.izpis()
        print
        print "Stevilo min v sosednjih celicah:"
        polje.izpis_sosednjih()
        print
        raw_input("Pritisni ENTER...")
    
    c.zacni(polje.w, polje.h, polje.N_min)
    c.readexpect('ZACEL')
    
    poteza, pohojene = 0, []
    while polje.neodkrito():
        poteza += 1
        c.potezaq()
        x,y = map(int, c.readexpect('POTEZA'))
        assert x >= 0 and x < polje.w and y >= 0 and y < polje.h, "Igralec poskusa odpreti celico izven igralnega polja: " + str(x) + "," + str(y)
        mina, sosednjih, fail = polje.pohodi(x, y)
        assert not fail, "Igralec poskusa odpreti ze odprto celico: " + str(x) + "," + str(y)
        if mina: pohojene.append(poteza)
        c.izid(x, y, 1 if mina else 0, sosednjih)
        c.readexpect('MHM')

    c.konec('pohodil_si_'+str(len(pohojene))+'_min')
    c.readexpect('KONCAL')
    return poteza, len(pohojene), pohojene

def main(client_prog, opt_N, opt_W, opt_H, opt_M, opt_R, opt_fnm, opt_dir, opt_polje_dump, opt_dump_prefix, debug_comm=False):
    if not opt_R: opt_R = time.time()
    random.seed(opt_R)
    if opt_N: print 'PRNG seme =', opt_R
    
    server = Socket(debug_comm)
    server.listen(1180)
    
    if client_prog: subprocess.Popen(client_prog)
    
    print 'Cakam na igralca ...'
    client, cli_ime = server.accept()
    print '... povezal se je igralec "%s".' % cli_ime
    
    vir = None
    if opt_N: vir = vir_rand
    elif opt_fnm: vir = vir_fnm
    elif opt_dir: vir = vir_dir
    else:
        raise Exception("Ni igralnih polj.")
    
    log = []
    for polje in vir(N=opt_N, W=opt_W, H=opt_H, M=opt_M, fnm=opt_fnm, dir=opt_dir, opt_pfx=opt_dump_prefix):
        N_potez, N_pohojenih, pohojene = igra(client, cli_ime, polje, opt_polje_dump)
        print "V", N_potez, "potezah je igralec pohodil", N_pohojenih, "min ob korakih", str(pohojene)
        log.append((N_potez, N_pohojenih, pohojene))
    client.adijo()

    if len(log) > 0:
        print 'Igralec je pohodil', sum(x[1] for x in log), 'min, v povprecju ', float(sum(x[1] for x in log)) / len(log), 'na igro.'

CMDLINE_HELP = """
Sintaksa:
  mine_test [-i <Igralec>] [-k] [-p] -n <SteviloIger> -w <Sirina>
             -h <Visina> -m <SteviloMin> [-s <Seme>] [-o <Predpona>]
  mine_test [-i <Igralec>] [-k] [-p] -f <Datoteka>
  mine_test [-i <Igralec>] [-k] [-p] -d <Direktorij>
  
  Parametri:
    -i <Igralec>     -- Pot do programa igralca. Ce je podan ta parameter,
                        ocenjevalni program samodejno zazene igralca, sicer
                        pa caka da zazenes igralca lastnorocno.
    -k               -- Izpisovanje komunikacije med ocenjevalnim programom in igralcem
    -p               -- Pred vsako igro izpise polje in pocaka
   Samodejno generiranje nakljucnih minskih polj:
    -n <SteviloIger> -- Stevilo nakljucnih iger, ki naj jih odigra igralec
    -w <Sirina>      -- Sirina igralnega polja
    -h <Visina>      -- Visina igralnega polja
    -m <SteviloMin>  -- Stevilo min na polju
    -s <Seme>        -- Seme za generator nakljucnih stevil
    -o <Predpona>    -- zapisi N-to igralno polje v datoteko "predpona_N.mp" 
   Branje minskega polja iz datoteke:
    -f <Datoteka>    -- Pot do datoteke v kateri je igralno polje
   Branje minskih polj iz direktorija
    -d <Direktorij>  -- Pot do direktorija v katerem so datoteke s polji (*.mp)
"""

def findopt(opts, name, typ=None):
    try:
        p = [o[1] for o in opts if o[0]=='-' + name][0]
        if typ: p = typ(p)
        return p
    except: return None

if __name__ == '__main__':
    print "Ocenjevalni program za Minolovca."
    print " http://rtk.ijs.si/2011/mine/"
    
    try:
        assert len(sys.argv) > 2
        
        opts,unparsed = getopt.getopt(sys.argv[1:], "n:w:h:f:d:m:s:i:kpo:")
        assert unparsed==[], "Neznan parameter: " + ', '.join(unparsed)
        
        opt_client_prog = findopt(opts, 'i')
        opt_N = findopt(opts, 'n', int)
        opt_W = findopt(opts, 'w', int)
        opt_H = findopt(opts, 'h', int)
        opt_M = findopt(opts, 'm', int)
        opt_R = findopt(opts, 's', int)
        opt_fnm = findopt(opts, 'f')
        opt_dir = findopt(opts, 'd')
        opt_dump_prefix = findopt(opts, 'o')
        opt_comm_debug = any(o[0]=='-k' for o in opts)
        opt_polje_dump = any(o[0]=='-p' for o in opts)
        
    except:
        #print traceback.print_exc()
        print CMDLINE_HELP
        sys.exit(1)
    
    try:
        main(opt_client_prog, opt_N, opt_W, opt_H, opt_M, opt_R, opt_fnm, opt_dir, opt_polje_dump, opt_dump_prefix, opt_comm_debug)
    except Exception, e:
        #print traceback.print_exc()
        print e
